import "Turbine";

local function AddCallback(object, event, callback)
    if (object == nil) then
        error("First argument to AddCallback() is nil", 2);
    else
        if (object[event] == nil) then
            object[event] = callback;
        else
            if (type(object[event]) == "table") then
                table.insert(object[event], callback);
            else
                object[event] = {object[event], callback};
            end
        end
    end
    return callback;
end

-- digits = number of fractional digits wanted (default is 0)
local function round(x, digits)
    digits = 10 ^ (digits or 0);
    return math.floor(x * digits + 0.5) / digits;
end

-- This conversion function is largely copied from the WayPoint plugin.  Thanks, LunarWtr!
local function ParseLocation(location)
	for _, regexp in ipairs({
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
        -- not likely to match last --
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
        'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)'
    }) do
        local r, lx, ly, ox, oy, oz, h = location:match(regexp);
        if (r) then
            lx, ly, ox, oy, oz, h = tonumber(lx), tonumber(ly), tonumber(ox), tonumber(oy), tonumber(oz), tonumber(h or 0);
            local x = (( math.floor( lx / 8 ) * 160 + ox ) - 29360) / 200;
            local y = (( math.floor( ly / 8 ) * 160 + oy ) - 24880) / 200;
            local coords = string.format("%.2f", math.abs(y)) .. (((y < 0) and "S") or "N") .. " | "
                .. string.format("%.2f", math.abs(x)) .. (((x < 0) and "W") or "E");
            local heading = round(h, 0) .. "°";
            return coords, heading, tonumber(r), x, y, h;
        end
	end
end

local function ParseChat(_, args)
    if (args.ChatType == Turbine.ChatType.Standard) then
        local loc = string.match(args.Message, "^You are on .* at (.*). Game timestamp ");
        if (loc) then
            local coords, heading, r, x, y, h = ParseLocation(loc);
            Turbine.Shell.WriteLine("<rgb=#FFC0FF>Precise coordinates: " .. coords .. "</rgb>");
        end
    end
end

AddCallback(Turbine.Chat, "Received", ParseChat);

--ParseChat(_, { ChatType = Turbine.ChatType.Standard; Message = "You are at: r4 lx1477 ly612 ox117.61 oy97.16 oz345.67 h128.0" });
--63.114S | 0.988E